@@ -64,7 +64,7 @@ ncb_hig_dialog (GtkMessageType type, char *title, char *reason,
 	char *title_esc, *reason_esc;
 
 	if (reason == NULL)
-		g_warning (" called with reason == NULL");
+		g_warning ("ncb_hig_dialog called with reason == NULL");
 
 	title_esc = g_markup_escape_text (title, -1);
 	reason_esc = g_markup_escape_text (reason, -1);
@@ -347,14 +347,13 @@ insert_cd_request_cb (CDRecorder *cdrecorder,
 
 	parent = glade_xml_get_widget (xml, "cdr_dialog");
 	dialog = ncb_hig_dialog (GTK_MESSAGE_INFO,
-			_(msg), _(title), GTK_WINDOW (parent));
+			_(title), _(msg), GTK_WINDOW (parent));
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
 			NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 			GTK_RESPONSE_OK);
-	gtk_window_set_title (GTK_WINDOW (dialog), _(title));
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 	if (res == GTK_RESPONSE_CANCEL) {
@@ -836,6 +835,7 @@ target_optionmenu_create (void)
 
 	widget = bacon_cd_selection_new ();
 	g_object_set (widget, "file-image", TRUE, NULL);
+	g_object_set (widget, "show-recorders-only", TRUE, NULL);
 	gtk_widget_show (widget);
 
 	return widget;
@@ -1149,7 +1149,7 @@ main (int argc, char *argv[])
 	}
 
 	if (g_file_test ("cdburn.glade", G_FILE_TEST_EXISTS) != FALSE) {
-		xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
+		xml = glade_xml_new ("./cdburn.glade", NULL, NULL);
 	} else {
 		xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
 	}
